@@ -16,6 +16,10 @@
 
 """cubicweb-worker specific hooks and operations"""
 
+from warnings import warn
+
+from logilab.common.registry import ObjectNotFound
+
 from cubicweb import ValidationError
 from cubicweb.predicates import is_instance
 from cubicweb.server.hook import Hook
@@ -72,17 +76,33 @@ class ShutdownWorker(Hook):
                     session.commit()
 
 
-class WorkerTaskOpcheck(Hook):
+def operation(klass, uopname):
+    try:
+        opname = 'do_%s' % uopname.encode('ascii')
+    except:
+        warn('Bad operation name %r' % uopname)
+        return
+    return getattr(klass, opname, None)
+
+
+class TaskPerformerOpcheck(Hook):
     __regid__ = 'worker.check_operation'
     events = ('after_add_entity',)
     __select__ = Hook.__select__ & is_instance('CWWorkerTask')
 
     def __call__(self):
         task = self.entity
-        adapter = task.cw_adapt_to(task.operation)
-        if adapter is None:
+        try:
+            performer = self._cw.vreg['worker.performer'].select(task.operation,
+                                                                 self._cw, task)
+        except ObjectNotFound:
+            # try bw compat. 1
+            adapter = task.cw_adapt_to(task.operation)
+            if adapter:
+                return
+            # bw compat. 2
             workerclass = self._cw.vreg['etypes'].etype_class('CWWorker')
-            if getattr(workerclass, 'do_%s' % task.operation, None):
+            if operation(workerclass, task.operation):
                 return
             msg = self._cw._('The operation name must match a Task '
                              'adapter of the same regid')
